      subroutine swe_model(u, v, phi, n, m, nt, k_first, k_start, k_end)

!     *************************************************************************
!     Solve the shallow water equations for the initial conditions given 
!     in u(:,:,1), v(:,:,1), phi(:,:,1).
! 
!     For more information, see "The Second Order Adjoint Analysis: Theory and 
!     Applications."  Wang, Z., Navon, I. M., Le Dimet F. X., and Zou X.  
!     Meteorology and Atmospheric Physics, Volume 50, Issue 1-3, Pages 3-20, 1992.  
!     DOI: 10.1007/BF01025501
!
!     as well as the paper "A Survey of Finite-Difference Schemes for 
!     the Primitive Equations for a Barotropic Fluid", 
!     Grammeltvedt, A.  Monthly Weather Review, Vol. 97,
!     Issue 5, Page 384, 1969.  
!     DOI: 10.1175/1520-0493(1969)097<0384:ASOFSF>2.3.CO;2
!     *************************************************************************

      implicit none

! size parameters
      integer, intent(in) :: n, m, nt, k_first, k_start, k_end

! state variables
      double precision, intent(inout), dimension(n, m, k_first:k_end) :: u, v, phi

! domain length and depth in m
      double precision :: length, depth, h0, h1, h2

! Namelist for initial condition parameter data
      namelist /icdata/ length, depth, h0, h1, h2

! Coriolis in 1/s
      double precision :: f
      data f/1.d-4/ 

! local variables
      double precision :: dx,dy,hx,hy,hhx,hhy,dt
      double precision :: aa,bb,cc,dd,phix,phiy,fu,fv,phiu,phiv
      double precision :: ztemp, htemp
      integer :: i,im1,ip1
      integer :: j,jm1,jp1
      integer :: k,kp1,kpass
      integer :: nm1,mm1,nm2,mm2

! read IC parameters from namelist
      open(unit=42,file="namelist.swm",status='old',form='formatted')
      read(42,icdata)
      close(42)

! time step variables, in s
      ztemp = 300.
      htemp = 2.*ztemp

! finite difference variables
      dx = (2.*length)/(n-1)
      dy = (2.*depth)/(m-1)
      hx = 1./(4.*dx)
      hy = 1./(4.*dy)
      hhx = 2.*hx
      hhy = 2.*hy

! convenience variables
      nm1 = n - 1
      mm1 = m - 1
      nm2 = nm1 - 1
      mm2 = mm1 - 1

! loop over time steps
      do 300 k=k_start-1,k_end-1
          kp1 = k+1

          if (k.gt.0)then
!         only run dynamics after the initial time step 
              if(k.eq.1)then
                  kpass=k
                  dt=ztemp
              else
                  kpass=k-1
                  dt=htemp
              endif

              do 200 j=2,mm1
                  do 210 i=2,nm1
                      ip1=i+1
                      im1=i-1
                      jp1=j+1
                      jm1=j-1

                      aa = (u(ip1,j,k) + u(im1,j,k))* &
                           (u(ip1,j,k) - u(im1,j,k))*hx

                      bb = (v(i,jp1,k) + v(i,jm1,k))* &
                           (u(i,jp1,k) - u(i,jm1,k))*hy

                      cc = (u(ip1,j,k) + u(im1,j,k))* &
                           (v(ip1,j,k) - v(im1,j,k))*hx

                      dd = (v(i,jp1,k) + v(i,jm1,k))* &
                           (v(i,jp1,k) - v(i,jm1,k))*hy

                      phix = (phi(ip1,j,k) - phi(im1,j,k))*hhx
                      phiy = (phi(i,jp1,k) - phi(i,jm1,k))*hhy

                      fu = f*u(i,j,k)
                      fv = f*v(i,j,k)

                      phiu = (phi(ip1,j,k)*u(ip1,j,k) - &
                              phi(im1,j,k)*u(im1,j,k))*hhx
                      phiv = (phi(i,jp1,k)*v(i,jp1,k) - &
                              phi(i,jm1,k)*v(i,jm1,k))*hhy

                      u(i,j,kp1) = u(i,j,kpass) - &
                                   dt*(aa + bb + phix - fv)

                      v(i,j,kp1) = v(i,j,kpass) - &
                                   dt*(cc + dd + phiy + fu)

                      phi(i,j,kp1) = phi(i,j,kpass) - dt*(phiu + phiv)
210               continue
200           continue
          endif

          call set_boundary_conditions(kp1, m, n, u(:,:,kp1), v(:,:,kp1), phi(:,:,kp1))

300   continue
      return
      end

subroutine set_boundary_conditions(k, m, n, u, v, phi)
    implicit none
    integer, intent(in) :: k
    integer, intent(in) :: m
    integer, intent(in) :: n
    double precision, dimension(n, m, k:k), intent(inout) :: u, v, phi
    integer :: i
    integer :: mm1,nm1

    nm1 = n - 1
    mm1 = m - 1

    do i=2,mm1
    !         left boundary 
        u(  1,i,k) = u(  nm1,i,k)
        v(  1,i,k) = v(  nm1,i,k)
        phi(1,i,k) = phi(nm1,i,k)
    enddo

    do i=1,n
    !         top boundary 
        u(  i,m,k) = u(  i,mm1,k)
        v(  i,m,k) = 0.
        phi(i,m,k) = phi(i,mm1,k)
    enddo

    do i=2,mm1
    !         right boundary 
        u(  n,i,k) = u(  2,i,k)
        v(  n,i,k) = v(  2,i,k)
        phi(n,i,k) = phi(2,i,k)
    enddo

    do i=1,n
    !         bottom boundary 
        u(i,1,k)   = u(i,2,k)
        v(i,1,k)   = 0.
        phi(i,1,k) = phi(i,2,k)
    enddo
    
!     top-right corner
    u(n,m,k) = (u(nm1,m,k) + u(n,mm1,k) + u(nm1,mm1,k))/3.
    v(n,m,k) = (v(nm1,m,k) + v(n,mm1,k) + v(nm1,mm1,k))/3.
    phi(n,m,k) = (phi(nm1,m,k) + phi(n,mm1,k) + phi(nm1,mm1,k))/3.
    
!     bottom-right corner
    u(n,1,k) = (u(nm1,1,k) + u(n,2,k) + u(nm1,2,k))/3.
    v(n,1,k) = (v(nm1,1,k) + v(n,2,k) + v(nm1,2,k))/3.
    phi(n,1,k) = (phi(nm1,1,k) + phi(n,2,k) + phi(nm1,2,k))/3.
    
!     bottom-left corner
    u(1,1,k) = (u(1,2,k) + u(2,1,k) + u(2,2,k))/3.
    v(1,1,k) = (v(1,2,k) + v(2,1,k) + v(2,2,k))/3.
    phi(1,1,k) = (phi(1,2,k) + phi(2,1,k) + phi(2,2,k))/3.
    
!     top-left corner
    u(1,m,k) = (u(1,mm1,k) + u(2,m,k) + u(2,mm1,k))/3.
    v(1,m,k) = (v(1,mm1,k) + v(2,m,k) + v(2,mm1,k))/3.
    phi(1,m,k) = (phi(1,mm1,k) + phi(2,m,k) + phi(2,mm1,k))/3.

end subroutine
